package Ajax.servlet;

import Ajax.pojo.User;
import com.alibaba.fastjson2.JSON;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 用户管理Servlet
 */
@WebServlet("/user.do")
public class UserServlet extends HttpServlet {
    @Override
    /**
     * 初始化用户数据
     */
    public void init() throws ServletException {
        ArrayList<User> list = new ArrayList<>();
        User user1 = new User("1", "zzq", "123", "2002-02-09", 2000.00);
        User user2 = new User("2", "张小东", "456", "1992-08-16", 12000.00);
        User user3 = new User("3", "哈哈哈", "789", "1980-02-19", 3000.00);
        User user4 = new User("4", "赵方兵", "123", "1999-04-07", 4000.00);
        User user5 = new User("5", "赵江枫", "123", "2000-05-06", 14000.00);
        list.add(user1);
        list.add(user2);
        list.add(user3);
        list.add(user4);
        list.add(user5);
        ServletContext servletContext=this.getServletContext();
        servletContext.setAttribute("userList",list);
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        this.doPost(req,resp);
    }

    /**
     * 取出list数据 将其转换成JSON数据
     * @param req
     * @param resp
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String flag = req.getParameter("flag");
        if ("getData".equals(flag)){
            this.getData(req,resp);
        }else if ("addUser".equals(flag)){
            this.addUser(req,resp);
        }else if ("updateUser".equals(flag)){
            this.updateUser(req,resp);
        }else if ("delete".equals(flag)){
            this.deleteUser(req,resp);
        }
    }

    /**
     * 删除指定用户
     * @param req
     * @param resp
     */
    private void deleteUser(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        ServletContext servletContext=this.getServletContext();
        ArrayList<User> list=(ArrayList<User>) servletContext.getAttribute("userList");
        String userId=req.getParameter("userid");
        Map<String,User> map=list.stream().collect(Collectors.toMap(User::getId,Function.identity()));
        User user=map.get(userId);
        System.out.println(user.toString());
        resp.setContentType("text/plain;charset=utf-8");
        PrintWriter pw=resp.getWriter();
        if (StringUtils.isNotEmpty(userId)){
            list.remove(user);
            System.out.println(list);
            pw.write("删除成功");
        }else{
            pw.write("删除失败");
        }
        pw.flush();
        pw.close();
    }

    private void updateUser(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        User user=this.createUser(req);
        ServletContext servletContext=this.getServletContext();
        ArrayList<User> list=(ArrayList<User>) servletContext.getAttribute("userList");
        //list转map  主要是为了获得要更新的user对象 最好还是写一个getUserById()方法，返回user对象
        Map<String,User> map=list.stream().collect(Collectors.toMap(User::getId, Function.identity()));
        //根据id获取指定的user
        User user1=map.get(user.getId());
        //删除指定的user
        list.remove(user1);
        //添加指定的user
        list.add(user);
        //按照id排序
        list.sort(new Comparator<User>() {
            @Override
            public int compare(User o1, User o2) {
                return Integer.parseInt(o1.getId())-Integer.parseInt(o2.getId());
            }
        });
        resp.setContentType("text/plain;charset=utf-8");
        PrintWriter pw = resp.getWriter();
        pw.print("更新成功");
        pw.flush();
        pw.close();
    }

    /**
     * 添加用户
     * @param req
     * @param resp
     */
    private void addUser(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        User user = this.createUser(req);
        ServletContext servletContext = this.getServletContext();
        ArrayList<User> list = (ArrayList<User>) servletContext.getAttribute("userList");
        System.out.println("user "+user);
        if (user!=null&&list!=null){
            list.add(user);
        }
        resp.setContentType("text/plain;charset=utf-8");
        PrintWriter pw = resp.getWriter();
        pw.print("添加成功");
        pw.flush();
        pw.close();
    }

    /**
     * 返回一个用户对象
     * @param req
     * @return
     */
    private User createUser(HttpServletRequest req) {
        String userid = req.getParameter("userid");
        String userName = req.getParameter("userName");
        String password = req.getParameter("password");
        String salary = req.getParameter("salary");
        String birthday = req.getParameter("birthday");
        User user = new User();
        user.setId(userid);
        user.setUserName(userName);
        user.setPassword(password);
        if (salary!=null){
            user.setSalary(Double.parseDouble(salary));
        }
        user.setBirthday(birthday);
        System.out.println(user);
        return user;
    }

    private void getData(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        ServletContext servletContext=this.getServletContext();
        ArrayList<User> list = (ArrayList<User>) servletContext.getAttribute("userList");
        String list_json=JSON.toJSONString(list);
        resp.setContentType("application/json");
        PrintWriter pw=resp.getWriter();
        pw.print(list_json);
        pw.flush();
        pw.close();
    }
}
